@@ -16,7 +16,7 @@ sub import {
 
 sub dont_succeed {
 
-    $DIE_LINE = __FILE__ . ' line ' . (__LINE__ + 2);   # Keywords introduce OB1 line reporting error
+    $DIE_LINE = __FILE__ . ' line ' . (__LINE__ + 1);
     fail "Didn't succeed";
 
     return 'This value should never be returned';
